@@ -22,7 +22,7 @@ class MessageTrace(TimeTrace, CatHeaderMixin):
 
         super(MessageTrace, self).__init__(transaction)
 
-        settings = self.transaction and self.transaction.settings or None
+        self.settings = self.transaction and self.transaction.settings or None
 
         if self.transaction:
             self.library = self.transaction._intern_string(library)
@@ -35,8 +35,8 @@ class MessageTrace(TimeTrace, CatHeaderMixin):
         # Only record parameters when not high security mode and only
         # when enabled in settings.
 
-        if (self.should_record_segment_params and settings and
-                settings.message_tracer.segment_parameters_enabled):
+        if (self.should_record_segment_params and self.settings and
+                self.settings.message_tracer.segment_parameters_enabled):
             self.params = params
         else:
             self.params = None
